﻿using System;
using System.Linq;

namespace AvlF8101.F8101
{
    public class AleCallTypes
    {
        public AleCallTypes()
        {
            ItemsSource = new IAleCallType[] 
            { 
                new Individual(), 
                new NetCall(), 
                new IndividualAmd(), 
                new NetAmd(), 
                new Sounding(), 
                new IndividualUpdateLqa(), 
                new NetUpdateLqa(),
                new UnlinkIndividualAmd(),
                new UnlinkNetAmd()
            };
        }

        public IAleCallType[] ItemsSource { get; private set; }
    }

    public interface IAleCallType
    {
        string Name { get; set; }

        byte[] MakeCommand(string id, string message);
    }

    abstract public class AleCallType : IAleCallType
    {
        public string Name { get; set; }

        abstract public byte[] MakeCommand(string id, string message);

    }

    public class Individual : AleCallType
    {
        public Individual()
        {
            Name = "Individual";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x00 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .ToArray();
        }
    }

    public class NetCall : AleCallType
    {
        public NetCall()
        {
            Name = "NET Call";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x01 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .ToArray();
        }
    }

    public class IndividualAmd : AleCallType
    {
        public IndividualAmd()
        {
            Name = "Individual AMD";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x02 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .Concat(new byte[] { 0x2C })
                .Concat(System.Text.Encoding.ASCII.GetBytes(message))
                .ToArray();
        }
    }

    public class NetAmd : AleCallType
    {
        public NetAmd()
        {
            Name = "NET AMD";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x03 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .Concat(new byte[] { 0x2C })
                .Concat(System.Text.Encoding.ASCII.GetBytes(message))
                .ToArray();
        }
    }

    public class Sounding : AleCallType
    {
        public Sounding()
        {
            Name = "Sounding";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x04 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .ToArray();
        }
    }

    public class IndividualUpdateLqa : AleCallType
    {
        public IndividualUpdateLqa()
        {
            Name = "Individual Update LQA";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x05 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .ToArray();
        }
    }

    public class NetUpdateLqa : AleCallType
    {
        public NetUpdateLqa()
        {
            Name = "NET Update LQA";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x06 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .ToArray();
        }
    }

    public class UnlinkIndividualAmd : AleCallType
    {
        public UnlinkIndividualAmd()
        {
            Name = "Unlink Individual AMD";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x07 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .Concat(new byte[] { 0x2C })
                .Concat(System.Text.Encoding.ASCII.GetBytes(message))
                .ToArray();
        }
    }

    public class UnlinkNetAmd : AleCallType
    {
        public UnlinkNetAmd()
        {
            Name = "Unlink NET AMD";
        }

        override public byte[] MakeCommand(string id, string message)
        {
            return (new byte[] { 0x08 })
                .Concat(System.Text.Encoding.ASCII.GetBytes(id))
                .Concat(new byte[] { 0x2C })
                .Concat(System.Text.Encoding.ASCII.GetBytes(message))
                .ToArray();
        }
    }
}
