﻿using System;
using System.IO.Ports;
using System.Collections.Generic;

namespace AvlF8101.F8101
{
    public class RxLog
    {
        public RxLog()
        {
        }

        public delegate void TextReceivedHandler(string value);

        public TextReceivedHandler TextReceived = delegate(string value) { };

        public delegate void PositionReceivedHandler(string id, double latitude, double longitude);

        public PositionReceivedHandler PositionReceived = delegate(string id, double latitude, double longitude) { };

        private List<byte> ReceiveBuffer = new List<byte>();

        public void AnalizeReceiveData(byte[] data)
        {
            ReceiveBuffer.AddRange(data);
            if (1000 < ReceiveBuffer.Count)
            {
                ReceiveBuffer.Clear();
            }

            for (string text = AnalyzeText(); text != null; text = AnalyzeText())
            {
                TextReceived(text);
                AnalyzeLog(text);
            }
        }

        private string AnalyzeText()
        {
            int stx = ReceiveBuffer.IndexOf(0x02);
            int etx = ReceiveBuffer.IndexOf(0x03);
            if ((stx < 0) || (etx <= stx + 1))
            {
                return null;
            }
            string log = System.Text.Encoding.ASCII.GetString(ReceiveBuffer.GetRange(stx + 1, etx - stx - 1).ToArray());
            ReceiveBuffer.RemoveRange(0, etx + 1);
            return log;
        }

        private void AnalyzeLog(string text)
        {
            if (0 > text.IndexOf("NTF,SEL,RXMSG,POS,"))
            {
                return;
            }
            if (0 < text.IndexOf("No GPS"))
            {
                return;
            }
            string[] param = text.Split(',');
            if (param.Length < 8)
            {
                return;
            }
            PositionReceived(param[5], AnalizeLat(param[6]), AnalizeLng(param[7]));
        }

        private double AnalizeLat(string text)
        {
            bool minus = false;
            int index = text.IndexOf('N');
            if (index < 0)
            {
                index = text.IndexOf('S');
                minus = true;
            }
            double value = double.Parse(text.Substring(0, index));
            return minus ? value * -1 : value;
        }

        private double AnalizeLng(string text)
        {
            bool minus = false;
            int index = text.IndexOf('E');
            if (index < 0)
            {
                index = text.IndexOf('W');
                minus = true;
            }
            double value = double.Parse(text.Substring(0, index));
            return minus ? value * -1 : value;
        }
    }
}
