﻿using System;
using System.Linq;

namespace AvlF8101.F8101
{
    public class SelcallCallTypes
    {
        public SelcallCallTypes()
        {
            ItemsSource = new ISelcallCallType[] 
            { 
                new Selective(), 
                new Phone(), 
                new Message(), 
                new SendPosition(), 
                new GetPosition(), 
                new GetStatus(), 
                new Emergency(),
                new ChannelTest(),
                new Stun()
            };
        }

        public ISelcallCallType[] ItemsSource { get; private set; }
    }

    public interface ISelcallCallType
    {
        string Name { get; set; }

        byte[] MakeCommand(string id, string phone, string pin, string message);
    }

    abstract public class SelcallCallType : ISelcallCallType
    {
        public string Name { get; set; }

        abstract public byte[] MakeCommand(string id, string phone, string pin, string message);

        protected byte[] toBcd(string id)
        {
            byte[] result = new byte[] { 0xFF, 0xFF, 0xFF, 0xFF, 0xFF };
            for (int i = 0; i < id.Length; i++)
            {
                int num = id[i] - '0';
                if (i % 2 == 0)
                {
                    result[i / 2] &= 0x0F;
                    result[i / 2] += (byte)(num << 4);
                }
                else
                {
                    result[i / 2] &= 0xF0;
                    result[i / 2] += (byte)num;
                }
            }
            return result;
        }
    }

    public class Selective : SelcallCallType
    {
        public Selective()
        {
            Name = "Selective";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x00 })
                .Concat(toBcd(id))
                .ToArray();
        }
    }

    public class Phone : SelcallCallType
    {
        public Phone()
        {
            Name = "Phone";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x01 })
                .Concat(toBcd(id))
                .Concat(System.Text.Encoding.ASCII.GetBytes(phone))
                .ToArray();
        }
    }

    public class Message : SelcallCallType
    {
        public Message()
        {
            Name = "Message";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x02 })
                .Concat(toBcd(id))
                .Concat(System.Text.Encoding.ASCII.GetBytes(message))
                .ToArray();
        }
    }

    public class SendPosition : SelcallCallType
    {
        public SendPosition()
        {
            Name = "Send Position";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x03 })
                .Concat(toBcd(id))
                .ToArray();
        }
    }

    public class GetPosition : SelcallCallType
    {
        public GetPosition()
        {
            Name = "Get Position";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x04 })
                .Concat(toBcd(id))
                .ToArray();
        }
    }

    public class GetStatus : SelcallCallType
    {
        public GetStatus()
        {
            Name = "Get Status";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x05 })
                .Concat(toBcd(id))
                .ToArray();
        }
    }

    public class Emergency : SelcallCallType
    {
        public Emergency()
        {
            Name = "Emergency";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x06 })
                .Concat(toBcd(id))
                .ToArray();
        }
    }

    public class ChannelTest : SelcallCallType
    {
        public ChannelTest()
        {
            Name = "Channel Test";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x07 })
                .Concat(toBcd(id))
                .ToArray();
        }
    }

    public class Stun : SelcallCallType
    {
        public Stun()
        {
            Name = "Stun";
        }

        override public byte[] MakeCommand(string id, string phone, string pin, string message)
        {
            return (new byte[] { 0x08 })
                .Concat(toBcd(id))
                .Concat(System.Text.Encoding.ASCII.GetBytes(pin))
                .ToArray();
        }
    }
}
