﻿using System;
using System.Windows;

using System.IO.Ports;
using AvlF8101.F8101;

namespace AvlF8101
{
    /// <summary>
    /// MainWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            _avl = new Avl(DispatchInvokeScript, DispatchInvokeMessageBox);
            _logWindow = new LogWindow();
            _logWindow.DataContext = _avl;
            _logWindow.Show();
            _civWindow = new CivWindow();
            _civWindow.DataContext = _avl;
            _civWindow.Show();
        }

        public void InvokeScript(string scriptName, params object[] args)
        {
            this._webBrowser.InvokeScript(scriptName, args);
        }

        public void DispatchInvokeScript(string scriptName, params object[] args)
        {
            Dispatcher.BeginInvoke(
                System.Windows.Threading.DispatcherPriority.ApplicationIdle, new Action(() => { InvokeScript(scriptName, args); }));
        }

        public void DispatchInvokeMessageBox(string message)
        {
            Dispatcher.BeginInvoke(
                System.Windows.Threading.DispatcherPriority.ApplicationIdle, new Action(() => { MessageBox.Show(message); }));
        }

        private Avl _avl;

        private void OpenCom_Checked(object sender, RoutedEventArgs e)
        {
            _avl.Open();
        }

        private void OpenCom_Unchecked(object sender, RoutedEventArgs e)
        {
            _avl.Close();
        }

        private void SettingCom_Click(object sender, RoutedEventArgs e)
        {
            OpenCom.IsChecked = false;
            ComSettingWindow dialog = new ComSettingWindow();
            dialog.Port = _avl.Port;
            dialog.BaudRate = _avl.BaudRate;
            dialog.DataBits = _avl.DataBits;
            dialog.Parity = _avl.Parity;
            dialog.StopBits = _avl.StopBits;
            if (dialog.ShowDialog() == true)
            {
                _avl.Port = dialog.Port;
                _avl.BaudRate = dialog.BaudRate;
                _avl.DataBits = dialog.DataBits;
                _avl.Parity = dialog.Parity;
                _avl.StopBits = dialog.StopBits;
                OpenCom.IsChecked = true;
            }
        }

        private LogWindow _logWindow;

        private CivWindow _civWindow;
    }
}
