﻿using System;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Collections.ObjectModel;
using System.Windows.Data;

namespace AvlF8101
{
    [ComVisibleAttribute(true)]
    public class MapHost : INotifyPropertyChanged
    {
        public void OnMarkerAdded(string id, string latitude, string longitude)
        {
            (this._markers.Source as ObservableCollection<Marker>).Add(
                new Marker() { Id = id, Latitude = ddToDmm(latitude), Longitude = ddToDmm(longitude), Time = DateTime.Now.ToString() });
            this.NotifyPropertyChanged("Markers");
        }

        private CollectionViewSource _markers = new CollectionViewSource() { Source = new ObservableCollection<Marker>() };

        public ICollectionView Markers
        {
            get
            {
                return this._markers.View;
            }
        }
        
        public void OnMarkerMoved(string id, string latitude, string longitude)
        {
            foreach (var item in (this._markers.Source as ObservableCollection<Marker>))
            {
                if (item.Id == id)
                {
                    item.Latitude = ddToDmm(latitude);
                    item.Longitude = ddToDmm(longitude);
                    break;
                }
            }
        }

        private string ddToDmm(string position)
        {
            double pos = double.Parse(position);
            return (((int)pos * 100) + ((pos % 1) * 60)).ToString();
        }

        private Marker _selectedMarker;

        public Marker SelectedMarker
        {
            get
            {
                return _selectedMarker;
            }
            set
            {
                _selectedMarker = value;
                this.NotifyPropertyChanged("SelectedMarker", "IsSelectedMarker");
            }
        }

        public bool IsSelectedMarker
        {
            get
            {
                if (_selectedMarker == null)
                {
                    return false;
                }
                return (_selectedMarker.Id != "");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(params string[] names)
        {
            if (this.PropertyChanged != null)
            {
                foreach (string name in names)
                {
                    this.PropertyChanged(this, new PropertyChangedEventArgs(name));
                }
            }
        }
    }

    public class Marker : INotifyPropertyChanged
    {
        private string _id;
        
        public string Id {
            get 
            {
                return _id; 
            }
            set
            {
                _id = value;
                this.NotifyPropertyChanged("Id");
            }
        }

        private string _latitude;

        public string Latitude {
            get
            {
                return _latitude;
            }
            set
            {
                _latitude = value;
                this.NotifyPropertyChanged("Latitude");
            }
        }

        private string _longitude;

        public string Longitude {
            get
            {
                return _longitude;
            }
            set
            {
                _longitude = value;
                this.NotifyPropertyChanged("Longitude");
            }
        }

        private string _time;

        public string Time
        {
            get
            {
                return _time;
            }
            set
            {
                _time = value;
                this.NotifyPropertyChanged("Time");
            }
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void NotifyPropertyChanged(params string[] names)
        {
            if (this.PropertyChanged != null)
            {
                foreach (string name in names)
                {
                    this.PropertyChanged(this, new PropertyChangedEventArgs(name));
                }
            }
        }
    }
}
