﻿using System;
using System.Windows.Controls;
using System.Windows;

namespace AvlF8101
{
    public static class WebBrowserAttachedProperty
    {
        public static string GetSource(DependencyObject obj)
        {
            return (string)obj.GetValue(SourceProperty);
        }

        public static void SetSource(DependencyObject obj, string value)
        {
            obj.SetValue(SourceProperty, value);
        }

        // Using a DependencyProperty as the backing store for Source.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty SourceProperty =
            DependencyProperty.RegisterAttached("Source", typeof(string), typeof(WebBrowserAttachedProperty), new PropertyMetadata(SourcePropertyChanged));

        public static void SourcePropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            WebBrowser browser = obj as WebBrowser;
            if (browser != null)
            {
                string uri = e.NewValue as string;
                browser.Source = uri != null ? new Uri(uri) : null;
            }
        }

        public static object GetObjectForScripting(DependencyObject obj)
        {
            return obj.GetValue(ObjectForScriptingProperty);
        }

        public static void SetObjectForScripting(DependencyObject obj, object value)
        {
            obj.SetValue(ObjectForScriptingProperty, value);
        }

        // Using a DependencyProperty as the backing store for DependencyObject.  This enables animation, styling, binding, etc...
        public static readonly DependencyProperty ObjectForScriptingProperty =
            DependencyProperty.RegisterAttached("ObjectForScripting", typeof(object), typeof(WebBrowserAttachedProperty), new PropertyMetadata(ObjectForScriptingPropertyChanged));

        public static void ObjectForScriptingPropertyChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            WebBrowser browser = obj as WebBrowser;
            if (browser != null)
            {
                browser.ObjectForScripting = e.NewValue;
            }
        }
    }
}
